@@ -1,4 +1,4 @@
-#if defined(__CYGWIN32__) || defined(USE_C_IO)
+#if defined(_WIN32) || defined(USE_C_IO)
 
 /*
  * Source: src/lib/rbeam18_cygwin32.c
@@ -68,18 +68,17 @@ void RBEAM18 (
     /* fix fortran filename */
     for(i = fname_len; fort_fname[i-1] == ' '; --i)
 	;
-    strncpy(fname, fort_fname, i+1);
+    strncpy(fname, fort_fname, i);
     fname[i] = '\0';
 
     *ierr = 0;
 
     fp = fopen(fname, "rb");
     if (!fp) {
-	i = 1;
-        mssg_ ("RBEAM18:: Error opening ", fort_fname, &i,
-            strlen("RBEAM18:: Error opening "), fname_len
-	);
 	*ierr = 1;
+        mssg_ ("RBEAM18:: Error opening ", fname, ierr,
+            strlen("RBEAM18:: Error opening "), i
+	);
 	return;
     }
     if (fread(io_dummy, sizeof(int), 5, fp) != 5) {
@@ -111,7 +110,7 @@ void RBEAM18 (
     return;
 }
 
-#else /* __CYGWIN32__ || USE_C_IO */
+#else /* _WIN32 || USE_C_IO */
 
 static int rbeam18_c_dummy;
 
