@@ -16,7 +16,10 @@
 #define excon_batch_job_h
 
 #include <sys/types.h>		// for pid_t definition
-#include "xrtypes.h"
+
+#if defined (_WIN32) && ! defined (__CYGWIN32__)
+  typedef int pid_t;
+#endif
 
 /**************************************************************************/
 
@@ -29,11 +32,11 @@ public:
     Job(const char* name, int jobid, pid_t pid);
     ~Job();
 
-    int getjobid()		{ return jobid_; }
-    pid_t getpid()		{ return pid_; }
-    const char* getname()	{ return name_; }
+    int getjobid() const	{ return jobid_; }
+    pid_t getpid() const	{ return pid_; }
+    const char* getname() const	{ return name_; }
 
-    int getstatus()		{ return status_; }
+    int getstatus() const	{ return status_; }
     void setstatus(int status)	{ status_ = status; }
 
 private:
