@@ -19,6 +19,7 @@
 
 #include "global.h"
 #include "link.h"
+#include "logger.h"
 #include "misc.h"
 #include "pmu.h"
 #include "tool.h"
@@ -33,13 +34,13 @@
  *
  ***************************************************************************/
 
-Tool::Tool (const char* name) {
+Tool::Tool(const char* name) {
     name_ = strcpy_with_alloc (name);
     vartable_ = new Hashtable (19); 
     varnlist_ = new Stq (); 
     links_ = new Stq ();
-    path_ = nil;
-    exec_ = nil;
+    path_ = 0;
+    exec_ = 0;
     runtime_ = 60;
 
     // add the PSEUDO variables that are automatically set.
@@ -52,62 +53,87 @@ Tool::Tool (const char* name) {
 }
 
 Tool::~Tool() {
-    delete[] name_ ; name_ = nil;
-    delete vartable_ ; vartable_ = nil;		// DELETE CONTAINER elems.
-    delete varnlist_; varnlist_ = nil;
+    delete[] name_ ; name_ = 0;
+    delete vartable_ ; vartable_ = 0;		// DELETE CONTAINER elems.
+    delete varnlist_; varnlist_ = 0;
 
     unsigned size = links_->size();
     while(size--) {
 	Link* link = (Link*)links_->pop();
-	delete link; link = nil;
+	delete link; link = 0;
     }
-    delete links_; links_ = nil;
+    delete links_; links_ = 0;
 
-    delete[] exec_ ; exec_ = nil;
-    delete[] path_ ; path_ = nil;
+    delete[] exec_ ; exec_ = 0;
+    delete[] path_ ; path_ = 0;
 }
 
-void Tool::errorcpuenv() { 
-    cerr 
-	<< "Could not find CPU environmental variable.\n"
-	<< "Set this variable and try again.\n";  
-    exit (1);   
+int Tool::nvars() const { 
+    return vartable_->size(); 
 }
 
-int Tool::nvars() const { return vartable_->size(); }
-
-void Tool::addvar(Var* var) { 
+int Tool::addvar(Var* var) { 
     vartable_->insert (var->getname(), var); 
     varnlist_->enq (var->getname());
+    return 0;
 }
 
 Var* Tool::getvar(const char* name) const { 
     return (Var*)vartable_->find (name); 
 }
 
-Var* Tool::getvar(int ith) const { return (Var*)vartable_->get(ith); }
-
-void Tool::setexec(const char* str) {
-    delete[] exec_ ; exec_ = nil;
-    exec_ = strcpy_with_alloc (str);
+Var* Tool::getvar(int ith) const { 
+    return (Var*)vartable_->get(ith); 
 }
 
-void Tool::setpath(const char* str) { 
-    // EXPAND ANY $??? VARIABLES
-    char buf1[1024];  char buf2[1024];  
-    strcpy (buf1, str);  
-    char* master = buf1; char* temp = buf2;  char* swap = nil;
+int Tool::setexec(const char* str) {
+    delete[] exec_;
+    exec_ = strcpy_with_alloc(str);
+    return 0;
+}
 
-    // SUBSTITUTE FOR CPU TYPE
-    if (strstr (buf1, "$CPU")) {
-	char* env = getenv ("CPU");  
-	if (env == nil) 
-	    errorcpuenv ();
-	strsub ("$CPU", env, master, temp); 
-	swap = master;  master = temp;  temp = swap;
+//
+// set the path of the program to run for the tool. Do expand the file
+// and search the path for it and issue warning if it's not found.
+//
+// Don't abort if not found, since we may not need to run the program
+// at all -- abort in genexec if we have to.
+//
+int Tool::setpath(const char* str) { 
+    char path[1024];
+    int retcode = 0;
+    switch(search_program(str, Globals::program_search_path, path)) {
+    case 0:
+	break;
+    
+    case 1:
+	EXCON_LOG_ALWAYS
+	    << "EXCON (Warning): "
+	    << "    Could not find Tool $PATH `" << str << "' in PATH."
+	    << endl;
+	retcode = 1;
+	break;
+    
+    case 2:
+	EXCON_LOG_ALWAYS
+	    << "EXCON: "
+	    << "    Error expanding Tool $PATH `" << str << "'."
+	    << endl
+	    << "    Check for environment variable or tilde expansion."
+	    << endl;
+	retcode = 1;
+	break;
+
+    case 3:
+	EXCON_LOG_ALWAYS
+	    << "EXCON: "
+	    << "    Tool $PATH program `" << str << "' is NOT executable!."
+	    << endl;
+	retcode = 1;
+	break;
     }
-    delete[] path_ ; path_ = nil;
-    path_ = strcpy_with_alloc (master);
+    path_ = strcpy_with_alloc(path);
+    return retcode;
 }
 
 /****************************************************************************
@@ -128,10 +154,10 @@ ToolInstance::ToolInstance (Tool* tool, int number, int iteration) {
     }
     instname_ = strcpy_with_alloc (buf);
     setiteration(iteration);
-    gpath_ = nil;
+    gpath_ = 0;
     // WARNING: DO NOT GENERATE AN exec LINE until right before running.
     // since the gfile depends on the iteration number
-    exec_ = nil;
+    exec_ = 0;
 
     // this is usually reset by client in ToolInstance::set_resolved_links
     resolved_links_ = new Stq;
@@ -146,23 +172,23 @@ ToolInstance::ToolInstance (Tool* tool, int number, int iteration) {
 
 
 ToolInstance::~ToolInstance() {
-    tool_ = nil; number_ = -1;
+    tool_ = 0; number_ = -1;
 
     unsigned size = instvars_->size();
     for(unsigned i = 1; i <= size; i++) {
 	Var* var = (Var*)instvars_->get(i);
-	delete var ; var = nil;
+	delete var ; var = 0;
     }
-    delete instvars_ ; instvars_ = nil;
+    delete instvars_ ; instvars_ = 0;
 
-    delete[] instname_ ; instname_ = nil;
-    delete[] gpath_ ; gpath_ = nil;
-    delete[] exec_; exec_ = nil;
+    delete[] instname_ ; instname_ = 0;
+    delete[] gpath_ ; gpath_ = 0;
+    delete[] exec_; exec_ = 0;
 
     size = resolved_links_->size();
     while(size--) {
 	ResolvedLink* link = (ResolvedLink*)resolved_links_->pop();
-	delete link; link = nil;
+	delete link; link = 0;
     }
     delete resolved_links_;
 
@@ -172,8 +198,9 @@ ToolInstance::~ToolInstance() {
     delete freevars_;
 }
 
-void ToolInstance::addvar(Var* var) { 
+int ToolInstance::addvar(Var* var) { 
     instvars_->insert(var->getname(), var); 
+    return 0;
 }
 
 // SIDE EFFECT: CHANGES PSEUDO VARIABLE VALUE.
@@ -184,14 +211,14 @@ void ToolInstance::setiteration(int iteration) {
     setvarval("$ITERATION", new IntegerValue (iteration));
 }
 
-void ToolInstance::setgpath() {
+int ToolInstance::setgpath() {
     char buf[1024];
     sprintf(buf, "%s+%d.g", instname_, iteration_);
-    delete[] gpath_ ; gpath_ = nil;
+    delete[] gpath_ ; gpath_ = 0;
     gpath_ = strcpy_with_alloc (buf); 
 
     // add the gfile name as a pseudo variable.
-    setvarval("$GFILE", new TextValue (gpath_));
+    return setvarval("$GFILE", new TextValue (gpath_));
 }
 
 
@@ -220,7 +247,8 @@ void ToolInstance::setgpath() {
 //
 
 
-void ToolInstance::resolveunique() {
+int ToolInstance::resolveunique() {
+    int errors = 0;
     int nvars = tool_->nvars();
 
     // pre-compute the UNIQUE substitution string.
@@ -231,7 +259,7 @@ void ToolInstance::resolveunique() {
 	const Var* tool_var = tool_->getvar (i);
 	if (tool_var->getval()->isatext()) {
 	    const char* text = tool_var->getval()->gettext();
-	    if (strstr (text, "$UNIQUE") != nil) {
+	    if (strstr (text, "$UNIQUE") != 0) {
 		char subst[1024];
 		// PERFORM REPLACEMENT
 		(void) strsub("$UNIQUE", unique, text, subst); 
@@ -241,16 +269,23 @@ void ToolInstance::resolveunique() {
 		// variable, and NOT the tool variable, since $SAVE is
 		// always added to variables in expseq_;
 		if (!getvar(tool_var->getname())->hasprop("$SAVE")) {
-		    GLB_cleanupmgr->addunlink (subst, getiteration());
+		    Globals::cleanupmgr->addunlink (subst, getiteration());
 		}
 	    }
 	}
     }
+    return errors;
 }
 
-void ToolInstance::genexec() {
+int ToolInstance::genexec() {
     // EX: $PATH -g $GFILE
 
+    //
+    // first see if the program exists and is executable!.
+    //
+    if (!fileexecutable(tool_->getpath()))
+	return 1;
+
     char buf[1024];  buf[0] = '\0';
     char token[1024];
     const char* exec = tool_->getexec();
@@ -269,8 +304,18 @@ void ToolInstance::genexec() {
 	token[tokenpos] = '\0';
 	// IF ITS A KEY WORD SUBSTITUTE...OTHERWISE PASS-THRU
 	if (strcmp(token,"$PATH")==0) {
-	    // INSERT THE EXEC PATH HERE
-	    strcat (buf, tool_->getpath());  strcat(buf," ");
+	    // INSERT THE EXEC PATH HERE, But take care to quote is so
+	    // that executing shell (cf: JobMgr::execute_job) doesn't get
+	    // all screwed up.
+	    bool embedded_space_in_path = 
+		strchr(tool_->getpath(), ' ') || 
+		strchr(tool_->getpath(), '\t');
+	    if (embedded_space_in_path)
+		strcat (buf, "\"");
+	    strcat (buf, tool_->getpath());  
+	    if (embedded_space_in_path)
+		strcat (buf, "\"");
+	    strcat(buf," ");
 	    continue;
 	}
 	if (strcmp(token,"$ITERATION")==0) {
@@ -290,51 +335,56 @@ void ToolInstance::genexec() {
     }
     delete[] exec_;
     exec_ = strcpy_with_alloc (buf);
+    return 0;
 }
 
 Var* ToolInstance::getvar(const char* name) const {
    Var* v = (Var*)instvars_->find(name);
-   return (v != nil) ? v : tool_->getvar(name);
+   return (v != 0) ? v : tool_->getvar(name);
 }
 
 const Value* ToolInstance::getvarval(const char* name) const {
    const Var* var = getvar (name);  
-   return (var != nil) ? var->getval() : nil;
+   return (var != 0) ? var->getval() : 0;
 }
 
 // ASSIGN val TO THE VARIABLE NAMED name
 // WARNING: NO COPY OF val IS MADE...THE POINTER IS DIRECTLY ASSIGNED
-void ToolInstance::setvarval (const char* name, Value* val) {
+int ToolInstance::setvarval (const char* name, Value* val) {
     // IF var IS NOT IN instvars ADD IT
     Var* var = (Var*)instvars_->find(name);
-    if (var == nil) {
+    if (var == 0) {
 	// FIND VAR NAME IN DEFAULT TABLE SO WE CAN GET THE VAR TYPE
-	const Var* defvar = tool_->getvar(name);  assume (defvar != nil);
+	const Var* defvar = tool_->getvar(name);  assume (defvar != 0);
 	var = defvar->clone();
 	var->setval(val);
+	var->addprop("$OVERRIDDEN_VALUE");
 	instvars_->insert(var->getname(), (void*)var);
     }
     else {
 	// IF WE ARE HERE JUST ASSIGN THE NEW VALUE
 	var->setval(val);
     }
+    return 0;
 }
 
 void ToolInstance::set_resolved_links(Stq* newlinks) {
     unsigned size = resolved_links_->size();
     while(size--) {
 	ResolvedLink* link = (ResolvedLink*)resolved_links_->pop();
-	delete link; link = nil;
+	delete link; 
+	link = 0;
     }
     delete resolved_links_;
     resolved_links_ = newlinks;
 }
 
+
 // READ A A gfile (NAMELIST of var = val PAIRS) 
 // RETURN 1 IF OK; -1 IF fopen FAILED RETURN -1
 int ToolInstance::readgfile(const char* gfile) {
     FILE *f = fopen (gfile, "r");  
-    if (f == nil) 
+    if (f == 0) 
 	return -1;
     
     int line = 0;   
@@ -344,7 +394,7 @@ int ToolInstance::readgfile(const char* gfile) {
     char text2[1024]; 
     Pmu pmu;
     while (1) {
-	if (fgets (buf, sizeof(buf), f) == nil) 
+	if (fgets (buf, sizeof(buf), f) == 0) 
 	    break;  
 	line++;
 	if (pmu.isacomment(buf)) 
@@ -354,26 +404,37 @@ int ToolInstance::readgfile(const char* gfile) {
 	if (pmu.match (buf, "<ident>", "=", "<text>")) {
 	    pmu.gettextfield (1, text1);
 	    pmu.gettextfield (3, text2);
-	    if (GLB_debug) 
-		cerr <<  "   VARIABLE: " << text1 << "=" << text2 << "\n";
+	    EXCON_LOG2(parse) 
+		<<  "   VARIABLE: " << text1 << "=" << text2 << "\n";
 	    // ...MAKE SURE VARIABLE HAS BEEN DEFINED
 	    const Var* var = getvar(text1);
-	    if (var == nil) {
-		cerr 
-		    << "The variable `" << text1 << "' is not defined for tool"
-		    << " " << instname_ 
-		    << "\nMake sure `" << text1 << "' is defined in file.\n";
-		errors++;  
+	    if (var == 0) {
+		EXCON_LOG_ALWAYS 
+		    << "EXCON: "
+		    << "(" << gfile << ":" << line << ")" 
+		    << endl
+		    << "    The variable `" << text1 
+		    << "' is not defined for tool " << instname_  
+		    << endl 
+		    << "    Make sure `" << text1 
+		    << "' is defined in TOOLS file."
+		    << endl;
+		errors++; 
 		continue;
 	    }
 	    // make sure the value is of the right type.
 	    Value* newval = var->getval()->clone(text2);
-	    if(newval == nil) {
-		cerr 
-		    << "Unexpected value (" << text2 << ") assigned to variable"
-		    << " `" << text1 << "'\n"
-		    << "The variable `" << text1 << "' is of type "
-		    << "`" << var->gettype() << "'.\n";
+	    if(newval == 0) {
+		EXCON_LOG_ALWAYS 
+		    << "EXCON: "
+		    << "(" << gfile << ":" << line << ")" 
+		    << endl
+		    << "    Unexpected value (" << text2 
+		    << ") assigned to variable `" << text1 << "'"
+		    << endl
+		    << "    The variable `" << text1 << "' is of type "
+		    << "`" << var->gettype() << "'."
+		    << endl;
 		errors++;  
 		continue;
 	    }
@@ -383,27 +444,38 @@ int ToolInstance::readgfile(const char* gfile) {
 
 	if (pmu.match (buf, "<ident>", "=", "<nothing>")) {
 	    pmu.gettextfield (1, text1);
-	    if (GLB_debug) 
-		cerr <<  "   VARIABLE: " << text1 << "=" << "<nothing>" << "\n";
+	    EXCON_LOG2(parse) 
+		<<  "   VARIABLE: " << text1 << "=" << "<nothing>" << "\n";
 	    // ...MAKE SURE VARIABLE HAS BEEN DEFINED
 	    const Var* var = getvar(text1);
-	    if (var == nil) {
-		cerr 
-		    << "The variable `" << text1 << "' is not defined for tool"
-		    << " " << instname_ 
-		    << "\nMake sure `" << text1 << "' is defined in file.\n";
+	    if (var == 0) {
+		EXCON_LOG_ALWAYS 
+		    << "EXCON: "
+		    << "(" << gfile << ":" << line << ")" 
+		    << endl
+		    << "    The variable `" << text1 
+		    << "' is not defined for tool " << instname_ 
+		    << endl 
+		    << "    Make sure `" << text1 
+		    << "' is defined in TOOLS file."
+		    << endl;
 		errors++;  
 		continue;
 	    }
 	    // ...MAKE SURE VALUE MATCHES VARIABLE TYPE
 	    const char* value = "NONE SPECIFIED";
 	    Value* newval = var->getval()->clone(value);
-	    if (newval == nil) {
-		cerr 
-		    << "Unexpected value (" << value << ") assigned to variable"
-		    << " `" << text1 << "'\n"
-		    << "The variable `" << text1 << "' is of type "
-		    << "`" << var->gettype() << "'.\n";
+	    if (newval == 0) {
+		EXCON_LOG_ALWAYS 
+		    << "EXCON: "
+		    << "(" << gfile << ":" << line << ")" 
+		    << endl
+		    << "    Unexpected value (" << value 
+		    << ") assigned to variable `" << text1 << "'"
+		    << endl
+		    << "    The variable `" << text1 << "' is of type `"
+		    << var->gettype() << "'."
+		    << endl;
 		errors++;  
 		continue;
 	    }
@@ -412,11 +484,17 @@ int ToolInstance::readgfile(const char* gfile) {
 	}
 
 	// UNRECOGNIZABLE PATTERN 
-	cerr <<  "Unable to understand line " << line << "..." << buf << "\n";
+	EXCON_LOG_ALWAYS 
+	    << "EXCON: "
+	    << "(" << gfile << ":" << line << ")" 
+	    << endl
+	    << "    Unable to understand line " << line << "..." << buf
+	    << endl;
 	errors++;  
 	if (errors > 3) {
 	    // IF WE ARE HERE TOO MAY ERRORS
-	    cerr << "Too many errors in gfile `" << gfile << "'\n";
+	    EXCON_LOG_ALWAYS 
+		<< "EXCON: Too many errors in gfile `" << gfile << "'\n";
 	    exit (1);
 	}
     }
@@ -430,26 +508,26 @@ int ToolInstance::readgfile(const char* gfile) {
 int ToolInstance::writegfile () const {
     char buf[1024];
     FILE *f = fopen (gpath_,"w");  
-    if (f == nil) 
+    if (f == 0) 
 	return -1;
     // SCAN ALL VARIABLES KNOWN TO tool
     for (int i=1; i<=tool_->nvars(); i++) {
-	const Var* var = tool_->getvar(i);  assume (var != nil);
+	const Var* var = tool_->getvar(i);  assume (var != 0);
 	// DO NOT INCLUDE OUTPUT VARIABLES
 	if (var->hasprop("$OUTPUTS")) 
 	    continue;
 	// NOTE: DO NOT QUERY var DIRECTLY FOR A VALUE BECAUSE IT COULD 
 	// BE OVERRIDDEN BY THE INSTANCE VERSION...INSTEAD CALL getvarval
-	const Value* val = getvarval (var->getname());  assume (val != nil);
+	const Value* val = getvarval (var->getname());  assume (val != 0);
 	sprintf (buf,"%s = %s\n",var->getname(), val->getstrval());
 	fputs (buf,f);
     }
     fclose (f);
-    GLB_stats->editedinputfile();
+    Globals::stats->editedinputfile();
 
-    const Var* var = getvar("$GFILE"); assume(var != nil);
+    const Var* var = getvar("$GFILE"); assume(var != 0);
     if (!var->hasprop("$SAVE"))
-	GLB_cleanupmgr->addunlink (gpath_, iteration_);
+	Globals::cleanupmgr->addunlink (gpath_, iteration_);
     return 1;
 }
 
@@ -486,35 +564,38 @@ ToolsMgr::ToolsMgr() {
    tools_ = new Hashtable (17);
    toollist_ = new Stq;
    pmu_ = new Pmu;
-   current_tool_ = nil;
-   currentloadfile_ = nil;
+   current_tool_ = 0;
+   currentloadfile_ = 0;
 }
 
 ToolsMgr::~ToolsMgr() {
     unsigned size = tools_->size();
     for(unsigned i = 1; i <= size; i++) {
 	Tool* tool = (Tool*)tools_->get(i);
-	delete tool ; tool = nil;
+	delete tool ; tool = 0;
     }
-    delete tools_ ; tools_ = nil;
-    delete toollist_; toollist_ = nil;
-    delete pmu_ ; pmu_ = nil;
-    current_tool_ = nil;
+    delete tools_ ; tools_ = 0;
+    delete toollist_; toollist_ = 0;
+    delete pmu_ ; pmu_ = 0;
+    current_tool_ = 0;
 }
 
 void ToolsMgr::errormissingtool(const char* file) {
-   cerr 
-       << "The tools file `" << file << "' must begin with a $TOOL definition."
-       << "\nAdd a $TOOL line to the file and try again.\n";
-   exit (1);
+    EXCON_LOG_ALWAYS 
+	<< "EXCON: "
+	<< "The tools file `" << file 
+	<< "' must begin with a $TOOL definition." << endl
+        << "    Add a $TOOL line to the file and try again." << endl;
+    exit (1);
 }
 
 void ToolsMgr::errorfilenotfound(const char* file) {
-   cerr
-       << "Could not open the tools file `" << file << "'.\n"
-       << "The file may not exists or may you may not have read permission.\n"
-       << "Check the file and try again.\n";
-   exit (1);
+    EXCON_LOG_ALWAYS 
+	<< "EXCON: "
+	<< "Could not open the tools file `" << file << "'.\n"
+	<< "The file may not exists or may you may not have read permission.\n"
+	<< "Check the file and try again.\n";
+    exit (1);
 }
 
 Tool* ToolsMgr::gettool(const char* toolname) const { 
@@ -525,73 +606,73 @@ const Stq* ToolsMgr::gettools() const {
     return toollist_;
 }
 
-void ToolsMgr::loadinputs(FILE* f, int& line) {
+int ToolsMgr::loadinputs(FILE* f, int& line) {
+    int errors = 0;
     char text1[1024];  
     char text2[1024];  
     int i;  
     double d;
 
     // current_tool MUST BE SET SO WE CAN POPULATE ITS RECORD
-    if (current_tool_ == nil) 
+    if (current_tool_ == 0) 
 	errormissingtool (currentloadfile_);
-    if (GLB_debug) 
-	cerr << "...LOADING INPUTS\n";
-    while (fgets (buf_, sizeof(buf_), f) != nil) {
+    EXCON_LOG2(parse) << "...LOADING INPUTS\n";
+    while (fgets (buf_, sizeof(buf_), f) != 0) {
 	line++;
 	if (pmu_->isacomment(buf_)) continue;
 	if (pmu_->match (buf_,"<ident>",":","text",":","<text>")) {
 	    pmu_->gettextfield (1,text1);
 	    pmu_->gettextfield (5,text2);
-	    if (GLB_debug) 
-		cerr << text1 << ":text:" << text2 << "\n";
+	    EXCON_LOG2(parse) 
+		<< text1 << ":text:" << text2 << "\n";
 	    // SEMANTICS
 	    Value* val = new TextValue (text2);
 	    Var* var = new Var (text1,"text",val);
-	    var->addprop("$INPUTS");
+	    var->addprop("$INPUTS")->addprop("$DEFAULT_VALUE");
 	    current_tool_->addvar (var);
 	} 
 	else if (pmu_->match (buf_,"<ident>",":","real",":","<real>")) {
 	    pmu_->gettextfield (1,text1);
 	    d = pmu_->getrealfield (5);
-	    if (GLB_debug) 
-		cerr << text1 << ":real:" << d << "\n";
+	    EXCON_LOG2(parse) 
+		<< text1 << ":real:" << d << "\n";
 	    // SEMANTICS
 	    Value* val = new RealValue (d);
 	    Var* var = new Var (text1,"real",val);
-	    var->addprop("$INPUTS");
+	    var->addprop("$INPUTS")->addprop("$DEFAULT_VALUE");
 	    current_tool_->addvar (var);
 	}
 	else if (pmu_->match (buf_,"<ident>",":","boolean",":","<yn>")) {
 	    pmu_->gettextfield (1,text1);
 	    i = pmu_->getcodefield (5);
-	    if (GLB_debug) 
-		cerr << text1 << ":boolean:" << i << "\n";
+	    EXCON_LOG2(parse) 
+		<< text1 << ":boolean:" << i << "\n";
 	    // SEMANTICS
 	    Value* val = new BooleanValue (i);
 	    Var* var = new Var (text1,"boolean",val);
-	    var->addprop("$INPUTS");
+	    var->addprop("$INPUTS")->addprop("$DEFAULT_VALUE");
 	    current_tool_->addvar (var);
 	}
 	else if (pmu_->match (buf_,"<ident>",":","integer",":","<integer>")) {
 	    pmu_->gettextfield (1,text1);
 	    i = pmu_->getintfield (5);
-	    if (GLB_debug) 
-		cerr << text1 << ":integer:" << i << "\n";;
+	    EXCON_LOG2(parse) 
+		<< text1 << ":integer:" << i << "\n";;
 	    // SEMANTICS
 	    Value* val = new IntegerValue (i);
 	    Var* var = new Var (text1,"integer",val);
-	    var->addprop("$INPUTS");
+	    var->addprop("$INPUTS")->addprop("$DEFAULT_VALUE");
 	    current_tool_->addvar (var);
 	}
 	else if (pmu_->match (buf_,"<ident>",":","filename",":","<text>")) {
 	    pmu_->gettextfield (1,text1);
 	    pmu_->gettextfield (5,text2);
-	    if (GLB_debug) 
-		cerr << text1 << ":filename:" << text2 << "\n";
+	    EXCON_LOG2(parse) 
+		<< text1 << ":filename:" << text2 << "\n";
 	    // SEMANTICS
 	    Value* val = new TextValue (text2);
 	    Var* var = new Var (text1,"filename",val);
-	    var->addprop("$INPUTS");
+	    var->addprop("$INPUTS")->addprop("$DEFAULT_VALUE");
 	    current_tool_->addvar (var);
 	}
 	else {
@@ -599,18 +680,20 @@ void ToolsMgr::loadinputs(FILE* f, int& line) {
 	    break;
 	}
     }
+    return errors;
 }
 
 // EX: inspectrum:sync  :$GETVAR outspectrum
-void ToolsMgr::loadlinks(FILE* f, int& line) {
+int ToolsMgr::loadlinks(FILE* f, int& line) {
+    int errors = 0;
     char text1[1024];  char text2[1024];  char text3[1024];
 
     // current_tool MUST BE SET SO WE CAN POPULATE ITS RECORD
-    if (current_tool_ == nil) 
+    if (current_tool_ == 0) 
 	errormissingtool (currentloadfile_);
-    if (GLB_debug) 
-	cerr <<  "...LOADING LINKS\n";
-    while (fgets(buf_, sizeof(buf_), f) != nil) {
+    EXCON_LOG2(parse) 
+	<<  "...LOADING LINKS\n";
+    while (fgets(buf_, sizeof(buf_), f) != 0) {
 	line++;
 	if (pmu_->isacomment(buf_)) continue;
 	if (pmu_->match (
@@ -619,14 +702,15 @@ void ToolsMgr::loadlinks(FILE* f, int& line) {
 	    pmu_->gettextfield (1,text1);
 	    pmu_->gettextfield (3,text2);
 	    pmu_->gettextfield (6,text3);
-	    if (GLB_debug) cerr 
+	    EXCON_LOG2(parse) 
 		<<  "   " << text1 << ":" << text2 << ":$GETVAR "
 		<< text3 << "\n";
 	    
 	    // make sure that the link variable is defined.
 	    Var* var = current_tool_->getvar(text1);
-	    if (var == nil) {
-		cerr 
+	    if (var == 0) {
+		EXCON_LOG_ALWAYS 
+		    << "EXCON: "
 		    << "LoadLinks: bad link variable '" << text1 
 		    << "' for tool `" << current_tool_->getname() 
 		    << "'. Ignoring...\n";
@@ -645,15 +729,15 @@ void ToolsMgr::loadlinks(FILE* f, int& line) {
 	    pmu_->gettextfield (1,text1);
 	    pmu_->gettextfield (3,text2);
 	    pmu_->gettextfield (6,text3);
-	    if (GLB_debug) 
-		cerr 
-		    << "   " << text1 << ":" << text2 << ":$RUN "
-		    << text3 << "\n";
+	    EXCON_LOG2(parse) 
+		<< "   " << text1 << ":" << text2 << ":$RUN "
+		<< text3 << "\n";
 	    
 	    // make sure that the link variable is defined.
 	    Var* var = current_tool_->getvar(text1);
-	    if (var == nil) {
-		cerr 
+	    if (var == 0) {
+		EXCON_LOG_ALWAYS 
+		    << "EXCON: "
 		    << "LoadLinks: bad link variable '" << text1 
 		    << "' for tool `" << current_tool_->getname() 
 		    << "'. Ignoring...\n";
@@ -671,17 +755,19 @@ void ToolsMgr::loadlinks(FILE* f, int& line) {
 	    break;
 	}
     }
+    return errors;
 }
 
-void ToolsMgr::loadoutputs(FILE* f, int& line) {
+int ToolsMgr::loadoutputs(FILE* f, int& line) {
+    int errors = 0;
     char text1[1024];  char text2[1024];  char text3[1024];
 
     // current_tool MUST BE SET SO WE CAN POPULATE ITS RECORD
-    if (current_tool_ == nil) 
+    if (current_tool_ == 0) 
 	errormissingtool (currentloadfile_);
-    if (GLB_debug) 
-	cerr << "...LOADING OUTPUTS\n";
-    while (fgets (buf_, sizeof(buf_), f) != nil) {
+    EXCON_LOG2(parse) 
+	<< "...LOADING OUTPUTS\n";
+    while (fgets (buf_, sizeof(buf_), f) != 0) {
 	line++;
 	if (pmu_->isacomment(buf_)) continue;
 	if (pmu_->match (buf_,"<ident>",":","<ident>",":","$RUN","<text>")) {
@@ -689,13 +775,13 @@ void ToolsMgr::loadoutputs(FILE* f, int& line) {
 	    pmu_->gettextfield (1,text1);
 	    pmu_->gettextfield (3,text2);
 	    pmu_->gettextfield (6,text3);
-	    if (GLB_debug) cerr
+	    EXCON_LOG2(parse) 
 		<< "   " << text1 << ":" << text2 << ":$RUN "
 		<< text3 << "\n";
 	    // SEMANTICS
 	    Value* val = new TextValue (text3);
 	    Var* var = new Var (text1, text2, val);
-	    var->addprop("$OUTPUTS");  var->addprop("$RUN");
+	    var->addprop("$OUTPUTS")->addprop("$RUN");
 	    current_tool_->addvar(var);
 	}
 	else {
@@ -703,63 +789,78 @@ void ToolsMgr::loadoutputs(FILE* f, int& line) {
 	    break;
 	}
     }
+    return errors;
 }
 
-void ToolsMgr::load(const char* path, int nesting) {
-    char text1[1024]; 
-    int line = 0, errors = 0;
+int ToolsMgr::load(const char* path, int nesting) {
+    if (nesting == 0) {
+	EXCON_LOG2(load) 
+	    << "EXCON: LOADING TOOLS FILE `" << path << "'\n";
+    } else {
+	EXCON_LOG2(nested_load) 
+	    << "                   `" << path << "'\n";
+    }
 
-    currentloadfile_ = path;			// UNCONST
-    if (GLB_debug) 
-	cerr << "LOADING TOOLS FILE `" << path << "'\n";
     FILE* f = fopen (path, "r");  
-    if (f == nil) 
+    if (f == 0) 
 	errorfilenotfound (path);
-    while (1) {
-	if (fgets (buf_,sizeof(buf_),f) == nil) 
-	    break;  
-	line++;
 
+    int errors = 0;			// # errors in this file
+    currentloadfile_ = path;		// local copy of filename
+
+    for(int line = 1; fgets(buf_, sizeof(buf_), f); ++line) { 
 	// PUT ALL SUBROUTINE CALLS ARE FIRST AND LET THEM FALL THRU
 	if (pmu_->match (buf_,"$INPUTS",":"))
-	    loadinputs (f, line);
+	    errors += loadinputs(f, line);
+
 	if (pmu_->match (buf_,"$LINKS",":")) 
-	    loadlinks (f, line);
+	    errors += loadlinks(f, line);
+
 	if (pmu_->match (buf_,"$OUTPUTS",":")) 
-	    loadoutputs (f, line);
+	    errors += loadoutputs(f, line);
 
 	// ALL OTHER PATTERNS GO HERE
-	if (pmu_->isacomment(buf_)) continue;
+	if (pmu_->isacomment(buf_)) 
+	    continue;
+
+	char text1[1024]; 		// temporary buffer for scanning
 	if (pmu_->match (buf_,"$TOOL",":","<ident>")) {
 	    pmu_->gettextfield (3,text1);
-	    if (GLB_debug) 
-		cerr << "   $TOOL: " << text1 << "\n";
-	    // SEMANTICS
-	    // ...INSERT TOOL INTO TOOL TABLE
+	    EXCON_LOG2(parse) 
+		<< "   $TOOL: " << text1 << "\n";
+	    //
+	    // semantics:
+	    // ...Insert TOOL into TOOL table. If the tool already
+	    // exists, issue a warning and replace.
+	    //
 	    current_tool_ = new Tool(text1);
 	    tools_->insert(current_tool_->getname(), current_tool_);
 	    toollist_->enq(current_tool_);
 	}
 	else if (pmu_->match (buf_,"$PATH",":","<text>")) {
 	    pmu_->gettextfield (3,text1);
-	    if (GLB_debug) 
-		cerr << "   $PATH: " << text1 << "\n";
+	    EXCON_LOG2(parse) 
+		<< "   $PATH: " << text1 << "\n";
 	    // SEMANTICS
+	    //
+	    // ignore error returns from setpath, since we catch these
+	    // in genexec() when actually running the program.
+	    //
 	    current_tool_->setpath (text1);
 	}
 	else if (pmu_->match (buf_,"$RUN",":","<text>")) {
 	    pmu_->gettextfield (3,text1);
-	    if (GLB_debug) 
-		cerr <<  "   $RUN: " << text1 << "\n";
+	    EXCON_LOG2(parse) 
+		<<  "   $RUN: " << text1 << "\n";
 	    // SEMANTICS
-	    current_tool_->setexec (text1);
+	    errors += current_tool_->setexec (text1);
 	}
-	else if (pmu_->match (buf_,"$RUNTIME",":","<integer>")) {
-	    int runtime = pmu_->getintfield (3);
-	    if (GLB_debug) 
-	    cerr << "   $RUNTIME: " << runtime << "\n";
+	else if (pmu_->match (buf_,"$RUNTIME",":","<long>")) {
+	    long runtime = pmu_->getlongfield (3);
+	    EXCON_LOG2(parse) 
+		<< "   $RUNTIME: " << runtime << "\n";
 	    // SEMANTICS
-	    current_tool_->setruntime (runtime);
+	    errors += current_tool_->setruntime (runtime);
 	}
 	//
 	// scan for include files with both "<file>" and "file" formats.
@@ -776,25 +877,33 @@ void ToolsMgr::load(const char* path, int nesting) {
 	    pmu_->match (buf_,"include","<name>")
 	) {
 	    pmu_->gettextfield(2, text1);
-	    if (GLB_debug) 
-		cerr << "   $INCLUDE: " << text1 << "\n";
+	    EXCON_LOG2(parse) 
+		<< "   $INCLUDE: " << text1 << "\n";
 	    // search and load new file.
 	    char newpath[1024];
-	    switch(search_file(text1, GLB_file_search_path, newpath)) {
+	    switch(search_file(text1, Globals::file_search_path, newpath)) {
 		case 0:
 		    load(newpath, nesting+1);
 		    break;
 		
 		case 1:
-		    cerr << "Cannot find INCLUDE file " << text1 
-			<< " in standard search path (" << path 
-			<< ":" << line << ")." << endl;
+		    EXCON_LOG_ALWAYS 
+			<< "EXCON: "
+			<< "(" << path << ":" << line << ")" 
+			<< endl
+			<< "    Cannot find INCLUDE file `" << text1
+			<< "' in standard search path."
+			<< endl;
 		    ++errors;
 		    break;
 
 		case 2:
-		    cerr << "Error opening INCLUDE file " << text1 
-			<< " (" << path << ":" << line << ")." << endl;
+		    EXCON_LOG_ALWAYS 
+			<< "EXCON: "
+			<< "(" << path << ":" << line << ")" 
+			<< endl
+			<< "    Error opening INCLUDE file `" << text1 << "'"
+			<< endl;
 		    ++errors;
 		    break;
 		
@@ -804,17 +913,22 @@ void ToolsMgr::load(const char* path, int nesting) {
 	}
 	else {
 	    // UNRECOGNIZABLE PATTERN 
-	    cerr 
-		<<  "Ignoring unknown line " << line << " ... " 
-		<< buf_ << "\n";
+	    EXCON_LOG_ALWAYS 
+		<< "EXCON: "
+		<< "(" << path << ":" << line << ")" 
+		<< endl
+		<<  "    Ignoring unknown line `" << buf_ << "'"
+		<< endl;
 	    ++errors;
 	}
 	if (errors > 3) {
-	    cerr << "Too many errors.  Check your tools file.\n";
+	    EXCON_LOG_ALWAYS 
+		<< "EXCON: "
+		<< "Too many errors.  Check your tools file.\n";
 	    fclose (f);  
 	    exit (1);
 	}
     }
     fclose (f);
-    return;
+    return errors;
 }
